#!/bin/bash

################################################################################
# Inputs
################################################################################
DAEMON_MODE=${FALSE}
DAEMON_NAME=unset
OPERATION=unset

################################################################################
# Commands
################################################################################
CMD_CAT="/bin/cat"
CMD_KILL="/bin/kill"
CMD_RM="/bin/rm"

################################################################################
# Constants
################################################################################
TRUE="True"
False="False"

LOG_OUTPUT_STDOUT="stdout"

PKG_NAME="ivs-base"
VS_ROOT="/usr/share/${PKG_NAME}"
VS_VAR="${VS_ROOT}/var"
VS_RUN="${VS_VAR}/run"
VS_LOG="${VS_VAR}/log"

IVEC_SHELL="${VS_ROOT}/qvsd.sh"
IVEC_OP_EXEC_DAEMON="exec_daemon"

IVEC_DAEMON_OP_CHECK="check"
IVEC_DAEMON_OP_START="start"
IVEC_DAEMON_OP_STOP="stop"

IDLE_SHELL="${VS_ROOT}/script/idle.sh"
IDLE_OP_START="start"
IDLE_OP_STOP="stop"

################################################################################
# Log functions
################################################################################
print()
{
    local output=$1
    local msg=$2

    if [ x"${output}" == x"${LOG_OUTPUT_STDOUT}" ]; then
        printf "${msg}"
    else
        printf "${msg}" >> ${output}
    fi
}

elog() {
    local i
    local def_log_dir=/tmp/qvs
    local log_output

    if [ -e ${VS_LOG} ]; then
        log_output=${VS_LOG}/daemon.log
    else
        log_output=${LOG_OUTPUT_STDOUT}
    fi

    if [ -f ${log_file} ]; then
        print "${log_output}" "[$(date +"%d/%b/%Y:%H:%M:%S %z")]"
    fi

    for i in $@
    do
        print "${log_output}" " $i"
    done
    print "${log_output}" "\n"
}

################################################################################
# Commons
################################################################################
kill_proc() {
    local signal=$1
    local pids=($2)
    for pid in ${pids[@]}
    do
        elog "Kill PID:[${pid}] ($(ps -p ${pid} -o cmd=)))"
        ${CMD_KILL} ${signal} ${pid}
    done
}

kill_proc_by_pid()
{
    local pid_path=$1
    local signal=$2

    if [ x"${signal}" == x"" ]; then
        signal="-15"
    fi

    if [ ! -f ${pid_path} ]; then
        elog "${pid_path} does not exist"
        return
    fi
    kill_proc "${signal}" $(${CMD_CAT} ${pid_path})
}

remove()
{
    local target=$1
    if [ -d ${target} ]; then
        elog "Remove Dir: ${target}"
    else
        elog "Remove File: ${target}"
    fi
    ${CMD_RM} -rf ${target}
}

idle()
{
    local pid_name=$1
    local op=$2
    if [ x"${op}" == x"${IDLE_OP_START}" ]; then
        ${IDLE_SHELL} ${pid_name} &
    elif [ x"${op}" == x"${IDLE_OP_STOP}" ]; then
        kill_proc_by_pid ${VS_RUN}/${pid_name}
        remove ${VS_RUN}/${pid_name}
    fi
}

ivec_daemon()
{
    local daemon_name="$1"
    local operation="$2"
    "${IVEC_SHELL}" "${IVEC_OP_EXEC_DAEMON}" "${daemon_name}" "${operation}"
}

################################################################################
# Functions
################################################################################
check()
{
    local daemon_name="$1"
    ivec_daemon "${daemon_name}" "${IVEC_DAEMON_OP_CHECK}"
}

start()
{
    local daemon_name="$1"
    local daemon_mode=${2:-${FALSE}}
    local pid_name="${daemon_name}.pid"
    ivec_daemon "${daemon_name}" "${IVEC_DAEMON_OP_START}"
    if [ x"${daemon_mode}" == x"${TRUE}" ]; then
        idle "${pid_name}" "${IDLE_OP_START}"
    fi
}

stop()
{
    local daemon_name="$1"
    local daemon_mode=${2:-${FALSE}}
    local pid_name="${daemon_name}.pid"
    ivec_daemon "${daemon_name}" "${IVEC_DAEMON_OP_STOP}"
    if [ x"${daemon_mode}" == x"${TRUE}" ]; then
        idle "${pid_name}" "${IDLE_OP_STOP}"
    fi
}

main()
{
    local daemon_name="$1"
    local operation="$2"
    local daemon_mode="$3"
    case ${operation} in
        ${IVEC_DAEMON_OP_CHECK})
            check "${daemon_name}"
            ;;
        ${IVEC_DAEMON_OP_START})
            start "${daemon_name}" "${daemon_mode}"
            ;;
        ${IVEC_DAEMON_OP_STOP})
            stop "${daemon_name}" "${daemon_mode}"
            ;;
    esac
}

args=$(getopt -a -o d --long daemon -- "$@")
if [[ $? -gt 0 ]]; then
    exit 1
fi

eval set -- ${args}
while :
do
    case $1 in
        -d | --daemon)
            DAEMON_MODE=${TRUE}
            shift
            ;;
        --)
            shift
            break
            ;;
        *)
            elog "Unsported option: $1"
            exit 1
            ;;
    esac
done
DAEMON_NAME=$1
OPERATION=$2
main "${DAEMON_NAME}" "${OPERATION}" "${DAEMON_MODE}"
